from typing import List

from bluer_options.terminal import show_usage


def help_help(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@help",
            "<command>",
        ],
        "show help for <command>.",
        mono=mono,
    )



from bluer_ai.plugins.web.accessible import is_accessible
from bluer_ai.env import abcli_is_github_workflow


def test_url_is_accessible():
    success = is_accessible("void")
    assert not success

    url = "https://cnn.com" if abcli_is_github_workflow else "https://iribnews.ir"
    success = is_accessible(url)
    assert success



from bluer_objects.README.alias import list_of_aliases

from bluer_ai import NAME
from bluer_ai.README import aliases


docs = [
    {
        "path": "../..",
        "macros": {
            "aliases:::": list_of_aliases(NAME),
        },
    },
    {
        "path": "../docs",
    },
] + aliases.docs



from flask import Flask, request, abort
import os
import argparse

from blueness import module
from bluer_options import string
from bluer_objects import file
from bluer_objects import objects

from bluer_ai import env
from bluer_ai import NAME, ICON
from bluer_ai.host import signature

NAME = module.name(__file__, NAME)

app = Flask(__name__)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--port_receive",
    type=str,
    default=env.BLUER_AI_WEB_RECEIVE_PORT,
)
parser.add_argument(
    "--port_send",
    type=str,
    default=env.BLUER_AI_WEB_SEND_PORT,
)
args = parser.parse_args()

object_path = objects.object_path(args.object_name)

app.config["UPLOAD_FOLDER"] = object_path


@app.post("/save")
def save():
    note_title = (request.form.get("note_title") or "").strip()
    content = request.form.get("content") or ""

    if not note_title:
        abort(400, "note title is required")

    # basic safety: prevent path traversal like ../../etc/passwd
    if "/" in note_title or "\\" in note_title or ".." in note_title:
        abort(400, "invalid note title")

    if not file.save_text(
        objects.path_of(
            object_name=object_path,
            filename=f"notes/{note_title}.txt",
        ),
        [content],
        log=True,
    ):
        abort(400, "saving content failed.")

    return (
        f'✅ saved to <a href="http://{env.BLUER_AI_IP}:{args.port_send}/notes/{note_title}.txt">{note_title}</a> | <a href="../">go back ...</a>\n',
        200,
    )


@app.route("/")
def upload_form():
    success, html = file.load_text(
        file.absolute(
            "./app.html",
            file.path(__file__),
        )
    )
    if not success:
        return "❗️ app.html not found."

    form = "\n".join(html)

    for this, that in {
        "{icon}": ICON,
        "{IP}": env.BLUER_AI_IP,
        "{logo}": env.BLUER_AI_WEB_LOGO,
        "{note_title}": string.timestamp(),
        "{port_receive}": args.port_receive,
        "{port_send}": args.port_send,
        "{footer}": " | ".join(signature()),
        "{header}": (
            ""
            if args.object_name == "void"
            else " | ".join(
                objects.signature(
                    object_name=args.object_name,
                )
            )
        ),
        "{qrcode}": "https://kamangir-public.s3.ir-thr-at1.arvanstorage.ir/kulej/qrcode.png",
    }.items():
        form = form.replace(this, that)

    return form


@app.route("/upload", methods=["POST"])
def upload_file():
    file = request.files["file"]
    if file.filename:
        file.save(
            objects.path_of(
                object_name=args.object_name,
                filename=file.filename,
            ),
        )
        return 'uploaded. ✅ | <a href="../">upload another file ...</a>.'

    return '⚠️ file not found | <a href="../">go back ...</a>.'


app.run(
    host="0.0.0.0",
    port=int(args.port_receive),
)



import requests


def is_accessible(
    url,
    timeout: int = 3,
) -> bool:
    try:
        response = requests.get(
            url,
            timeout=timeout,
        )
        return response.status_code == 200
    except:
        return False



import argparse


from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_ai import NAME
from bluer_ai.plugins.web.status.get import get_status
from bluer_ai.plugins.web.status.loop import get_status_on_a_loop
from bluer_ai.plugins.web.accessible import is_accessible
from bluer_ai.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="identify | is_accessible",
)
parser.add_argument(
    "--count",
    type=int,
    default=-1,
    help="-1: forever",
)
parser.add_argument(
    "--log",
    type=int,
    default=-1,
    help="0 | 1",
)
parser.add_argument(
    "--loop",
    type=int,
    default=0,
    help="0 | 1",
)
parser.add_argument(
    "--object_name",
    type=str,
    default="",
)
parser.add_argument(
    "--sleep",
    type=int,
    default=5,
    help="in seconds",
)
parser.add_argument(
    "--timeout",
    type=int,
    default=3,
    help="in seconds",
)
parser.add_argument(
    "--timestamp",
    type=int,
    default=0,
    help="0 | 1",
)
parser.add_argument(
    "--url",
    type=str,
)
args = parser.parse_args()

success = False
if args.task == "is_accessible":
    success = True
    print(
        int(
            is_accessible(
                args.url,
                timeout=args.timeout,
            )
        )
    )
elif args.task == "identify":
    if args.loop:
        success = get_status_on_a_loop(
            timeout=args.timeout,
            log=args.log != 0,
            count=args.count,
            sleep=args.sleep,
            object_name=args.object_name,
            timestamp=args.timestamp == 1,
        )
    else:
        success = True
        print(
            get_status(
                timeout=args.timeout,
                log=args.log == 1,
                timestamp=args.timestamp == 1,
            )
        )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from bluer_objects import objects

from bluer_ai.plugins.web.status.loop import get_status_on_a_loop


def test_web_get_status_on_a_loop():
    object_name = objects.unique_object("test_web_get_status_on_a_loop")

    assert get_status_on_a_loop(
        object_name,
        timeout=3,
        log=False,
        count=3,
        sleep=5,
    )



import os

from bluer_options import string

from bluer_ai.modules.terraform.functions import poster
from bluer_ai import env


def test_terraform_poster():
    filename = os.path.join(
        env.ABCLI_PATH_IGNORE,
        "background{}.jpg".format(string.timestamp()),
    )
    assert poster(filename)



from typing import List

from bluer_options.terminal import show_usage, xtra

from bluer_ai import env


def help_share(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "download,~open,~receive,~send,upload"

    port_options = xtra(
        "port.receive.=<{}>,port.send=<{}>".format(
            env.BLUER_AI_WEB_RECEIVE_PORT,
            env.BLUER_AI_WEB_SEND_PORT,
        ),
        mono=mono,
    )

    return show_usage(
        [
            "@web",
            "share",
            f"[{options}]",
            f"[{env.BLUER_AI_WEB_OBJECT}|<object-name>]",
            f"[{port_options}]",
        ],
        "share <object-name>.",
        mono=mono,
    )



from typing import List

from bluer_options.terminal import show_usage

from bluer_ai import env


def help_open(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@web",
            "open",
        ],
        f"open the web object ({env.BLUER_AI_WEB_OBJECT}).",
        mono=mono,
    )



from typing import Dict
import time

from blueness import module
from bluer_options import string
from bluer_objects.metadata import post_to_object

from bluer_ai import NAME
from bluer_ai.plugins.web.status.get import get_status
from bluer_ai.logger import logger

NAME = module.name(__file__, NAME)


def get_status_on_a_loop(
    object_name: str,
    timeout: int = 3,
    log: bool = False,
    count: int = -1,
    sleep: int = 30,
    timestamp: bool = False,
) -> bool:
    logger.info(
        "{}.get_status_on_a_loop: timeout={}{}, sleep:{} -{}> {}".format(
            NAME,
            string.pretty_duration(timeout),
            "" if count == -1 else f", {count} sample(s)",
            string.pretty_duration(sleep),
            "timestamp-" if timestamp else "",
            object_name,
        )
    )

    list_of_status: Dict[float, str] = {}
    try:
        while True:
            list_of_status[string.pretty_date()] = get_status(
                timeout=timeout,
                log=log,
                timestamp=timestamp,
            )

            if count != -1 and len(list_of_status) > count:
                logger.info(f"{count} samples collected.")
                break

            time.sleep(sleep)
    except KeyboardInterrupt:
        logger.warning("Ctrl+C, stopping.")
    except Exception as e:
        logger.error(e)
        return False

    logger.info("collected {} sample(s).".format(len(list_of_status)))

    return post_to_object(
        object_name,
        "status",
        list_of_status,
    )



from typing import List

from bluer_options.terminal import show_usage

args = [
    "[--timeout 3]",
]


def help_is_accessible(
    tokens: List[str],
    mono: bool,
) -> str:

    return show_usage(
        [
            "@web",
            "is_accessible",
            "<url>",
        ]
        + args,
        "is <url> accessible?",
        mono=mono,
    )



from typing import List

from bluer_options.terminal import show_usage, xtra
from bluer_ai.help.web import is_accessible


def help_identify(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "loop",
            xtra(",~upload", mono=mono),
        ]
    )

    args = sorted(
        [
            "[--sleep <5>]",
            "[--timestamp 0]",
        ]
        + is_accessible.args
    )

    return show_usage(
        [
            "@web",
            "identify",
            f"[{options}]",
            "[-|<object-name>]",
        ]
        + args,
        "identify web connection.",
        mono=mono,
    )



from bluer_options import string

from bluer_ai import env
from bluer_ai.plugins.web.accessible import is_accessible
from bluer_ai.logger import logger

dict_of_status = {
    "national": "🇮🇷",
    "offline": "⛓️‍💥",
    "online": "🛜",
}


def get_status(
    timeout: int = 3,
    log: bool = False,
    timestamp: bool = False,
) -> str:
    status = (
        "offline"
        if not is_accessible(
            env.BLUER_AI_INTERNET_ONLINE_CHECK_URL,
            timeout=timeout,
        )
        else (
            "national"
            if not is_accessible(
                env.BLUER_AI_INTERNET_NATIONAL_CHECK_URL,
                timeout=timeout,
            )
            else "online"
        )
    )

    if log:
        logger.info(
            "{}{} {}".format(
                (
                    "{} ".format(
                        string.timestamp(
                            unique_length=0,
                        )
                    )
                    if timestamp
                    else ""
                ),
                dict_of_status.get(status, "❓"),
                status,
            )
        )

    return status



from typing import List

from bluer_options.terminal import show_usage


def help_get_ip(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@web",
            "get_ip",
        ],
        "get IP.",
        mono=mono,
    )



from bluer_options.testing import are_nonempty_strs

from bluer_ai import env


def test_bluer_ai_env():
    assert are_nonempty_strs(
        [
            env.BLUER_AI_INTERNET_NATIONAL_CHECK_URL,
            env.BLUER_AI_INTERNET_ONLINE_CHECK_URL,
            env.BLUER_AI_NATIONAL_INTERNET_INDEX,
            env.BLUER_AI_WEB_RECEIVE_PORT,
            env.BLUER_AI_WEB_SEND_PORT,
            env.BLUER_AI_WEB_OBJECT,
        ]
    )



from bluer_ai.help.web.get_ip import help_get_ip
from bluer_ai.help.web.identify import help_identify
from bluer_ai.help.web.is_accessible import help_is_accessible
from bluer_ai.help.web.open import help_open
from bluer_ai.help.web.share import help_share
from bluer_ai.help.web.where_am_i import help_where_am_i


help_functions = {
    "get_ip": help_get_ip,
    "identify": help_identify,
    "is_accessible": help_is_accessible,
    "open": help_open,
    "share": help_share,
    "where_am_i": help_where_am_i,
}
